# -*- coding: utf8 -*-

from django.conf import settings
from django.core.mail import send_mail, EmailMessage
from django.core.management.base import CommandError
from django.template.loader import render_to_string

from .models import a_photo_enreg, image_photo_enreg, invalidate_noms_prenoms
from ldap_inp import LdapEleve, LdapEleveAvecPhotoEtPwd

import unidecode
from portailuser.models import Ecole

GID_NORMALUSER = 1000
SHELL = "/bin/bash"


def prochain_uid_libre(min_value: int = 2000, max_value: int = 59999):
    if max_value > 60000:
        raise CommandError("Prochain uid supérieur à 60.000")
    avg_value = (min_value + max_value) // 2
    present = len(LdapEleve.objects.filter(uidNumber=str(avg_value))) != 0
    if min_value == max_value:
        if present:
            return None
        else:
            return avg_value

    if present:
        return prochain_uid_libre(avg_value + 1, max_value)
    else:
        return prochain_uid_libre(min_value, avg_value)


def envoyer_email(inscr, email):
    c = {
        "inscr": inscr,
    }

    message = render_to_string("gestion/inscriptions/mail_bienvenue.txt", c)
    message_admin = render_to_string("gestion/inscriptions/mail_bienvenue_admin.txt", c)
    EmailMessage(
        "Création de ton compte INP-net",
        message,
        "INP-net <noreply@bde.inp-toulouse.fr>",
        [f"{inscr.prenom} {inscr.nom} <{email}>"],
        headers={"Reply-To": "INP-net <inp-net@bde.inp-toulouse.fr>"},
    ).send(fail_silently=False)
    send_mail(
        "Compte Créé",
        message_admin,
        "INP-net <inp-net@bde.inp-toulouse.fr>",
        ["Inscription <inscription@bde.inp-toulouse.fr>"],
        fail_silently=False,
    )


def creer_compte(inscr, uid, noeud_base):
    print(f"Création de l’entrée LDAP pour l'utilisateur {uid}…")

    if inscr.ecole != Ecole.ensat:
        inscr.inscrit_frappe = True

    donnees = {
        "parent": noeud_base,
        "uid": inscr.login,
        "uidNumber": uid,
        "gidNumber": GID_NORMALUSER,
        "loginShell": SHELL,
        "userPassword": "!",
        "homeDirectory": "/home/" + unidecode.unidecode(inscr.login),
        "prenom": inscr.prenom,
        "nom": inscr.nom,
        "genre": inscr.genre,
        "birthdate": inscr.date_naissance,
        "promo": inscr.promo,
        "ecole": inscr.ecole,
        "filiere": inscr.filiere,
        "inscritAE": inscr.inscrit_ae,
        "inscritFrappe": inscr.inscrit_frappe,
        "inscritPassVieEtudiant": inscr.inscrit_passvie_etudiant,
        "finPassVieEtudiant": inscr.fin_inscrit_passvie_etudiant,
        "loginTP": inscr.ident_cri,
        "nickname": inscr.surnom,
        "mailAnnexe": [inscr.email] if inscr.email else [],
        "postalAddress": inscr.adresse,
        "parentsAddress": inscr.adr_parents,
        "mobile": inscr.tel_mobile,
        "homePhone": inscr.tel_perso,
        "mail": f"{unidecode.unidecode(inscr.login)}@{settings.EMAIL_BDE[inscr.ecole.id - 1]}",
        "mailForwardingAddress": inscr.mail_cri,
        "mailEcole": inscr.mail_cri,
        "sitePersoBDE": False,
        "jpegPhoto": image_photo_enreg(inscr.id) if a_photo_enreg(inscr.id) else None,
    }

    eleve = LdapEleveAvecPhotoEtPwd(**donnees)
    eleve.save(account="root")
    eleve.change_password(inscr.mdp)
    eleve.save()

    # l'inscription d'une nouvelle personne invalide le cache de noms/prénoms, sinon on risque d'inscrire
    # plusieures fois la même personne
    invalidate_noms_prenoms()

    print("Envoi du mail de bienvenue…")
    # On envoie le mail uniquement à l'adresse étudiante, pour empécher l'usurpation d'identité lors de la création de comptes
    envoyer_email(inscr, inscr.mail_cri)

    inscr.mdp = "!"
    inscr.save()
